### Encoding: latin1

### Name: ff
### Title: ff classes for representing (large) atomic data
### Aliases: ff ff_pointer
### Keywords: IO data array attribute classes package

### ** Examples

  cat("make sure you understand the following ff options before you start using the ff package!!\n")
  oldoptions <- options(fffinalizer="deleteIfOpen", fffinonexit="TRUE", fftempdir=tempdir())
  ff(1:12)                        # an integer vector
  ff(0, 12)                       # a double vector of length 12
  ff(vmode="logical", length=12)  # a logical vector of length 12 (due to NA using 2 bit per cell on disk, vmode="boolean" uses 1 bit)
  ff(1:12, dim=c(3,4))            # an integer matrix 3x4 (standard colwise physical layout)
  ff(1:12, dim=c(3,4), dimorder=c(2,1)) # an integer matrix 3x4 (rowwise physical layout, but filled in standard colwise order)
  ff(1:12, dim=c(3,4), bydim=c(2,1)) # an integer matrix 3x4 (standard colwise physical layout, but filled in rowwise order aka matrix(, byrow=TRUE))
  options(oldoptions)

  if (ffxtensions()){
     a <- ff(vmode="boolean", dim=rep(2, 26)) # a 26-dimensional boolean array using 1-bit representation (file size 8 MB compared to 256 MB int in ram)
     dimnames(a) <- dummy.dimnames(a)
  }

  ## Not run: 
##D 
##D      cat("This 2GB biglm example can take long, you might want to change the size in order to define a size appropriate for your computer\n")
##D      require(biglm)
##D 
##D      b <- 1000
##D      n <- 100000
##D      k <- 3
##D      memory.size(max = TRUE)
##D      system.time(
##D      x <- ff(vmode="double", dim=c(b*n,k), dimnames=list(NULL, LETTERS[1:k]))
##D      )
##D      memory.size(max = TRUE)
##D      system.time(
##D      ffrowapply({
##D         l <- i2 - i1 + 1
##D         z <- rnorm(l)
##D         for (i in 1:k)
##D           x[i1:i2,i] <- z + rnorm(l)
##D      }, X=x, VERBOSE=TRUE, BATCHSIZE=n)
##D      )
##D      memory.size(max = TRUE)
##D 
##D      form <- A ~ B + C
##D      first <- TRUE
##D      system.time(
##D      ffrowapply({
##D         if (first){
##D           first <- FALSE
##D           fit <- biglm(form, as.data.frame(x[i1:i2,,drop=FALSE]))
##D         }else
##D           fit <- update(fit, as.data.frame(x[i1:i2,,drop=FALSE]))
##D      }, X=x, VERBOSE=TRUE, BATCHSIZE=n)
##D      )
##D      memory.size(max = TRUE)
##D      first
##D      fit
##D      summary(fit)
##D   
## End(Not run)



